@@ -22,11 +22,147 @@
 
 ***********************************************************************************/
 
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <iomanip>
 #include <gtkmm/stock.h>
+
 #include "helpers.hpp"
 
+
 using namespace LIFEO;
 
+
+Error::Error( const Glib::ustring &error_message )
+{
+	print_error( error_message );
+}
+
+// DATE
+Glib::Date::Month
+Date::get_month_glib( void ) const
+{
+	switch( ( m_date & 0x78000 ) >> 15 )
+	{
+		case 1:
+			return Glib::Date::JANUARY;
+		case 2:
+			return Glib::Date::FEBRUARY;
+		case 3:
+			return Glib::Date::MARCH;
+		case 4:
+			return Glib::Date::APRIL;
+		case 5:
+			return Glib::Date::MAY;
+		case 6:
+			return Glib::Date::JUNE;
+		case 7:
+			return Glib::Date::JULY;
+		case 8:
+			return Glib::Date::AUGUST;
+		case 9:
+			return Glib::Date::SEPTEMBER;
+		case 10:
+			return Glib::Date::OCTOBER;
+		case 11:
+			return Glib::Date::NOVEMBER;
+		case 12:
+			return Glib::Date::DECEMBER;
+		default:
+			return Glib::Date::BAD_MONTH;
+	}
+}
+
+// TODO: make date string format user configurable
+Glib::ustring
+Date::format_string( void ) const
+{
+	return Glib::ustring::compose( "%1.%2.%3",
+			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), get_year() ),
+			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), get_month() ),
+			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), get_day() ) );
+}
+
+Glib::ustring
+Date::format_string( const time_t *time )
+{
+	struct tm *timeinfo = localtime( time );
+	return Glib::ustring::compose( "%1.%2.%3, %4:%5",
+			1900 + timeinfo->tm_year,
+			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), timeinfo->tm_mon + 1 ),
+			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), timeinfo->tm_mday ),
+			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), timeinfo->tm_hour ),
+			Glib::ustring::format( std::setfill( L'0' ), std::setw( 2 ), timeinfo->tm_min ) );
+}
+
+Glib::ustring
+Date::get_year_str( void ) const
+{
+	return Glib::ustring::compose( "%1", get_year() );
+}
+
+Glib::ustring
+Date::get_month_str( void ) const
+{
+	return Glib::Date( 20, get_month_glib(), 2000 ).format_string( "%b" );
+}
+
+Glib::ustring
+Date::get_weekday_str( void ) const
+{
+	// from wikipedia: http://en.wikipedia.org/wiki/Calculating_the_day_of_the_week
+	const unsigned int year = get_year();
+	const unsigned int century = ( year - ( year % 100 ) ) / 100;
+	int c = 2 * ( 3 - ( century % 4 ) );
+	int y = year % 100;
+	y = y + floor( y / 4 );
+
+	const static int tm[] = { 0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5 };
+	// FIXME: day names had better be taken from the system
+	const static Glib::ustring days[] = { "Sunday", "Monday", "Tuesday", "Wednesday",
+										  "Thursday", "Friday", "Saturday" };
+
+	int m = get_month() - 1;
+	int d = ( c + y + tm[ m ] + get_day() );
+
+	if( !( get_year() % 4 ) && m < 2 )	// leap year!
+		d += 6;
+
+	return days[ d % 7 ];
+}
+
+time_t
+Date::get_ctime( void ) const
+{
+	time_t t;
+	time( &t );
+	struct tm *timeinfo = localtime( &t );
+	timeinfo->tm_year = get_year() - 1900;
+	timeinfo->tm_mon = get_month() - 1;
+	timeinfo->tm_mday = get_day();
+	timeinfo->tm_hour = 0;
+	timeinfo->tm_min = 0;
+	timeinfo->tm_sec = 0;
+
+	return( mktime( timeinfo ) );
+}
+
+// CONSOLE MESSAGES
+void
+LIFEO::print_error( const Glib::ustring &description )
+{
+	std::cout << "ERROR: " << description << std::endl;
+}
+
+void
+LIFEO::print_info( const Glib::ustring &description )
+{
+	std::cout << "INFO: " << description << std::endl;
+}
+
 // STRING PROCESSING
 long
 LIFEO::convert_string( const std::string &str )
@@ -133,16 +269,51 @@ LIFEO::open_url( Gtk::AboutDialog&, const Glib::ustring &link )
 {
 	std::string command( "x-www-browser " );
 	command += link;
-	//TODO: what else can be made with the return value?
-	std::cout << system( command.c_str() ) << std::endl;
+	std::cout << "opening url: " << system( command.c_str() ) << std::endl;
+	// TODO: find a better way
 }
 
 void
 LIFEO::mail_to( Gtk::AboutDialog&, const Glib::ustring &link )
 {
+	// TODO: find a good way
 }
 
 // ENCRYPTION
+bool
+Cipher::init( void )
+{
+	// http://www.gnupg.org/documentation/manuals/gcrypt/Initializing-the-library.html
+
+	// initialize subsystems:
+	if( ! gcry_check_version( NULL ) )	// TODO: check version
+	{
+		print_error( "Libgcrypt version mismatch" );
+		return false;
+	}
+
+	// disable secure memory
+	gcry_control( GCRYCTL_DISABLE_SECMEM, 0 );
+
+	// MAYBE LATER:
+	/*
+	// suppress warnings
+	gcry_control( GCRYCTL_SUSPEND_SECMEM_WARN );
+
+	// allocate a pool of 16k secure memory. this makes the secure memory...
+	// ...available and also drops privileges where needed
+	gcry_control( GCRYCTL_INIT_SECMEM, 16384, 0 );
+     
+	// resume warnings
+	gcry_control( GCRYCTL_RESUME_SECMEM_WARN );
+	*/
+
+	// tell Libgcrypt that initialization has completed
+	gcry_control( GCRYCTL_INITIALIZATION_FINISHED, 0 );
+
+	return true;
+}
+
 void
 Cipher::create_iv( unsigned char **iv )
 {
@@ -240,7 +411,7 @@ Cipher::encrypt_buffer (	unsigned char *buffer,
 	error = gcry_cipher_open( &cipher, cCIPHER_ALGORITHM, cCIPHER_MODE, 0 );
 
 	if( error )
-		throw Error( "Unable to initilize cipher: " ); // + gpg_strerror( Error ) );
+		throw Error( "unable to initialize cipher: " ); // + gpg_strerror( Error ) );
 
 	// GET KEY LENGTH
 	int cipherKeyLength = gcry_cipher_get_algo_keylen( cCIPHER_ALGORITHM );
@@ -322,10 +493,40 @@ Cipher::decrypt_buffer (	unsigned char * buffer,
 	gcry_cipher_close( cipher );
 }
 
+// MARKED UP MENU ITEM
+Gtk::MenuItem*
+LIFEO::create_menuitem_markup(	const Glib::ustring &str_markup,
+								const Glib::SignalProxy0< void >::SlotType &handler )
+{
+	// thanks to GNote for showing the way
+	Gtk::MenuItem *menuitem = Gtk::manage( new Gtk::MenuItem( str_markup ) );
+	Gtk::Label *label = dynamic_cast< Gtk::Label* >( menuitem->get_child() );
+	if( label )
+		label->set_use_markup( true );
+	menuitem->signal_activate().connect( handler );
+	return menuitem;
+}
+
 // ENTRYIDLETEXT
+EntryIdletext::EntryIdletext( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& )
+	:	Gtk::Entry( cobject ), m_flag_empty( true )
+{
+	signal_icon_release().connect(
+			sigc::mem_fun( *this, &EntryIdletext::handle_clear_icon ) );
+}
+
 EntryIdletext::EntryIdletext( const Glib::ustring& idletext )
 : Gtk::Entry(), m_idletext( idletext ), m_flag_empty( true )
 {
+	signal_icon_release().connect(
+			sigc::mem_fun( *this, &EntryIdletext::handle_clear_icon ) );
+}
+
+void
+EntryIdletext::handle_clear_icon( Gtk::EntryIconPosition icon_pos, const GdkEventButton *event )
+{
+	if( icon_pos == Gtk::ENTRY_ICON_SECONDARY && event->button == 1 )
+		set_text( "" );
 }
 
 void
@@ -346,6 +547,12 @@ EntryIdletext::on_changed( void )
 {
 	m_flag_empty = get_text().empty();
 
+	if( m_flag_empty )
+		// Fall-back to C API as gtkmm has problems with this:
+		gtk_entry_set_icon_from_stock( gobj(), GTK_ENTRY_ICON_SECONDARY, 0 );
+	else
+		set_icon_from_stock( Gtk::Stock::CLEAR, Gtk::ENTRY_ICON_SECONDARY );
+
 	Gtk::Entry::on_changed();
 }
 
@@ -376,15 +583,18 @@ EntryIdletext::on_key_release_event( GdkEventKey *event )
 	return Gtk::Entry::on_key_release_event( event );
 }
 
-// MENUBUTTON
+// MENUBUTTON ==================================================================
 Menubutton::Menubutton(	const Gtk::StockID &stockid,
 						const Glib::ustring &label,
 						Gtk::ReliefStyle style,
-						Gtk::IconSize iconsize )
-	:	Gtk::ToggleButton()
+						Gtk::IconSize iconsize,
+						Menu2 *menu )
+	:	Gtk::ToggleButton(), m_menu( menu )
 {
 	set_relief( style );
-	m_menu = new Menu2;
+
+	if( menu == NULL )
+		m_menu = new Menu2;
 	m_hbox = Gtk::manage( new Gtk::HBox );
 	m_vbox = Gtk::manage( new Gtk::VBox );
 	m_icon = Gtk::manage( new Gtk::Image( stockid, iconsize ) );
@@ -404,23 +614,35 @@ Menubutton::Menubutton(	const Gtk::StockID &stockid,
 }
 
 //FIXME: code duplication
-Menubutton::Menubutton(	const std::string& path,
+Menubutton::Menubutton(	const Glib::RefPtr< Gdk::Pixbuf > *pixbuf,
 						const Glib::ustring& label,
-						Gtk::ReliefStyle style )
-	:	Gtk::ToggleButton()
+						Gtk::ReliefStyle style,
+						Menu2 *menu )
+	:	Gtk::ToggleButton(), m_menu( menu )
 {
 	set_relief( style );
-	m_menu = new Menu2;
-	m_hbox = Gtk::manage( new Gtk::HBox );
-	m_vbox = Gtk::manage( new Gtk::VBox );
-	m_icon = Gtk::manage( new Gtk::Image( path ) );
-	m_label = Gtk::manage( new Gtk::Label( label ) );
-	Gtk::Arrow *arrow = Gtk::manage(
-			new Gtk::Arrow( Gtk::ARROW_DOWN, Gtk::SHADOW_IN ) );
+	Gtk::Arrow *arrow;
+	try
+	{
 
-	m_hbox->pack_start(*m_vbox, Gtk::PACK_SHRINK);
+		if( menu == NULL )
+			m_menu = new Menu2;
+		m_hbox = Gtk::manage( new Gtk::HBox );
+		m_vbox = Gtk::manage( new Gtk::VBox );
+		if( pixbuf )
+			m_icon = Gtk::manage( new Gtk::Image( *pixbuf ) );
+		m_label = Gtk::manage( new Gtk::Label( label ) );
+		arrow = Gtk::manage( new Gtk::Arrow( Gtk::ARROW_DOWN, Gtk::SHADOW_IN ) );
+	}
+	catch( ... )
+	{
+		throw 0;	// TODO...
+	}
+
+	m_hbox->pack_start( *m_vbox, Gtk::PACK_SHRINK );
 	// the only purpose of Vbox is supporting Gtk::TOOLBAR_BOTH toolbar style
-	m_vbox->pack_start( *m_icon, Gtk::PACK_SHRINK );
+	if( pixbuf )
+		m_vbox->pack_start( *m_icon, Gtk::PACK_SHRINK );
 	m_hbox->pack_start( *m_label, Gtk::PACK_EXPAND_WIDGET );
 	m_hbox->pack_start( *arrow, Gtk::PACK_SHRINK );
 	add( *m_hbox );
@@ -434,12 +656,28 @@ Menubutton::~Menubutton( void )
 	delete m_menu;
 }
 
-Menu2*&
+Menu2*
 Menubutton::get_menu( void )
 {
 	return m_menu;
 }
 
+bool
+Menubutton::clear_menu( void )
+{
+	if ( m_menu->items().size() > 0 )
+	{
+		delete m_menu;
+		m_menu = new Menu2;
+		m_menu->attach_to_widget( *this );
+		m_menu->signal_deactivate().connect( sigc::mem_fun( *this, &Menubutton::release ) );
+
+		return true;
+	}
+	else
+		return false;
+}
+
 void
 Menubutton::release( void )
 {
@@ -473,48 +711,52 @@ Menubutton::on_button_press_event (GdkEventButton *l_event)
 }
 
 // MENUTOOLBUTTON
-Menutoolbutton::Menutoolbutton (const Gtk::StockID& stockid,
-								const std::string& string)
-	:	m_button( stockid, string, Gtk::RELIEF_NONE, Gtk::ICON_SIZE_LARGE_TOOLBAR )
+Menutoolbutton::Menutoolbutton( const Gtk::StockID& stockid,
+							    const std::string& string,
+							    Menu2 *menu,
+							    bool flag_important)
+:	m_flag_important( flag_important ),
+	m_button( stockid, string, Gtk::RELIEF_NONE, Gtk::ICON_SIZE_LARGE_TOOLBAR, menu )
 {
 	add( m_button );
+	m_button.m_icon->show();
+	m_button.m_label->show();
 }
 
-Menu2*&
+Menu2*
 Menutoolbutton::get_menu (void)
 {
 	return m_button.m_menu;
 }
 
-void Menutoolbutton::on_toolbar_reconfigured (void)
+void
+Menutoolbutton::on_toolbar_reconfigured (void)
 {
-	switch ( get_toolbar_style() )
+	if( m_button.m_label->get_parent() )
+		m_button.m_label->get_parent()->remove( *m_button.m_label );
+
+	if( m_button.m_icon->get_parent() )
+		m_button.m_icon->get_parent()->remove( *m_button.m_icon );
+
+	switch( get_toolbar_style() )
 	{
 		case Gtk::TOOLBAR_BOTH:
-			if( m_button.m_label->get_parent() == m_button.m_hbox )
-			{
-				m_button.m_hbox->remove( *m_button.m_label );
-				m_button.m_vbox->pack_start ( *m_button.m_label );
-			}
-			m_button.m_label->show();
-			m_button.m_icon->show();
+			m_button.m_vbox->pack_start( *m_button.m_icon );
+			m_button.m_vbox->pack_start( *m_button.m_label );
 		break;
 		case Gtk::TOOLBAR_BOTH_HORIZ:
-			if( m_button.m_label->get_parent() == m_button.m_vbox ) {
-				m_button.m_vbox->remove( *m_button.m_label );
-				m_button.m_hbox->pack_start ( *m_button.m_label );
-				m_button.m_hbox->reorder_child ( *m_button.m_label, 1 );
+			m_button.m_vbox->pack_start( *m_button.m_icon );
+			if( m_flag_important )
+			{
+				m_button.m_hbox->pack_start( *m_button.m_label );
+				m_button.m_hbox->reorder_child( *m_button.m_label, 1 );
 			}
-			m_button.m_label->show();
-			m_button.m_icon->show();
 		break;
 		case Gtk::TOOLBAR_ICONS:
-			m_button.m_label->hide();
-			m_button.m_icon->show();
+			m_button.m_vbox->pack_start( *m_button.m_icon );
 		break;
 		case Gtk::TOOLBAR_TEXT:
-			m_button.m_icon->hide();
-			m_button.m_label->show();
+			m_button.m_vbox->pack_start( *m_button.m_label );
 		break;
 		default:
 		break;
@@ -534,7 +776,7 @@ MenuitemRecent::MenuitemRecent( const std::string &path )
 	label->set_alignment( Gtk::ALIGN_LEFT );
 	label->set_ellipsize( Pango::ELLIPSIZE_START );
 	label->set_tooltip_text( path );
-	m_icon_remove.set_tooltip_text( _("Remove from list") );
+	m_icon_remove.set_tooltip_text( _( "Remove from the list" ) );
 	hbox->set_spacing( 5 );
 
 	hbox->pack_start( *label );
@@ -596,11 +838,13 @@ MenuitemRecent::on_button_release_event( GdkEventButton *event )
 
 
 // FILEBUTTONRECENT
-FilebuttonRecent::FilebuttonRecent( const Glib::ustring& label_fallback )
-	:	Menubutton( ICONDIR "/diary-16.png", label_fallback, Gtk::RELIEF_NORMAL ),
+FilebuttonRecent::FilebuttonRecent(	const Glib::RefPtr< Gdk::Pixbuf > &pixbuf,
+									ListPaths *list_recent,
+									const Glib::ustring& label_fallback )
+	:	Menubutton( &pixbuf, label_fallback, Gtk::RELIEF_NORMAL ),
 	m_icon_new( Gtk::Stock::NEW, Gtk::ICON_SIZE_MENU ),
 	m_icon_browse( Gtk::Stock::OPEN, Gtk::ICON_SIZE_MENU ),
-	m_recentcount( 0 )
+	m_list_recent( list_recent )
 {
 	m_hbox->set_spacing( 4 );
 
@@ -634,25 +878,14 @@ FilebuttonRecent::FilebuttonRecent( const Glib::ustring& label_fallback )
 std::string
 FilebuttonRecent::get_filename( void ) const
 {
-	return( *m_list_recent.begin() );
+	return( * m_list_recent->begin() );
 }
 
-//std::string
-//FilebuttonRecent::get_filename_recent (int index) const {
-//	if( index >= RecentCount )
-//		return "";
-//	else {
-//		std::list< std::string >::iterator l_iter;
-//		for ( int i = 0; i < index; i++ ) l_iter++;
-//		return (*l_iter);
-//	}
-//}
-
 void
 FilebuttonRecent::set_filename( const std::string &path )
 {
-	if( m_recentcount > 0 )
-		if( path == ( *m_list_recent.begin() ) )
+	if( ! m_list_recent->empty() )
+		if( path == * m_list_recent->begin() )
 			return;
 
 	m_label->set_label( Glib::filename_display_basename( path ) );
@@ -662,12 +895,28 @@ FilebuttonRecent::set_filename( const std::string &path )
 }
 
 void
-FilebuttonRecent::update_filename()
+FilebuttonRecent::update_filenames()
 {
-	if( m_recentcount > 0 )
+	if( ! m_list_recent->empty() )
 	{
-		m_label->set_label( Glib::filename_display_basename( *m_list_recent.begin() ) );
-		set_tooltip_text( *m_list_recent.begin() );
+		for( ListPaths::reverse_iterator iter = m_list_recent->rbegin();
+			 iter != m_list_recent->rend(); ++iter )
+		{
+			MenuitemRecent *menuitem = Gtk::manage( new MenuitemRecent( * iter ) );
+
+			menuitem->signal_activate().connect(
+					sigc::bind(	sigc::mem_fun( *this, &FilebuttonRecent::set_filename ),
+								* iter ) );
+			menuitem->signal_removerecent().connect(
+					sigc::mem_fun( *this, &FilebuttonRecent::remove_recent ) );
+
+			m_menu->items().push_front( Gtk::Menu_Helpers::Element( *menuitem ) );
+
+			menuitem->show();
+		}
+
+		m_label->set_label( Glib::filename_display_basename( * m_list_recent->begin() ) );
+		set_tooltip_text( * m_list_recent->begin() );
 	}
 }
 
@@ -676,21 +925,17 @@ FilebuttonRecent::add_recent( const std::string &path )
 {
 	// do not add a file twice:
 	Gtk::Menu_Helpers::MenuList::iterator iter_menu = m_menu->items().begin();
-	std::list< std::string >::iterator iter_recent = m_list_recent.begin();
-	int i;
-	for( i = 0; i < m_recentcount; i++ )
+	ListPaths::iterator iter_recent = m_list_recent->begin();
+	for( ; iter_recent != m_list_recent->end(); ++iter_recent )
 	{
 		if( ( *iter_recent ) == path )
 		{
 			m_menu->items().erase( iter_menu );
-			m_list_recent.erase( iter_recent );
+			m_list_recent->erase( iter_recent );
 			break;
 		}
 		else
-		{
-			iter_recent++;
-			iter_menu++;
-		}
+			++iter_menu;
 	}
 
 	MenuitemRecent *menuitem = Gtk::manage( new MenuitemRecent( path ) );
@@ -705,15 +950,12 @@ FilebuttonRecent::add_recent( const std::string &path )
 
 	menuitem->show();
 
-	m_list_recent.push_front( path );
+	m_list_recent->push_front( path );
 
-	// if nothing is deleted:
-	if( i == m_recentcount )
+	if( m_list_recent->size() > MAX_RECENT_FILE_COUNT )
 	{
-		if( m_recentcount == MAX_RECENT_FILE_COUNT )
-			m_menu->items().erase( --iter_menu );
-		else
-			m_recentcount++;
+		m_menu->items().erase( --iter_menu );
+		m_list_recent->erase( --iter_recent );
 	}
 
 	return true;	// reserved
@@ -723,28 +965,28 @@ void
 FilebuttonRecent::remove_recent( const std::string &path )
 {
 	Gtk::Menu_Helpers::MenuList::iterator	iter_menu = m_menu->items().begin();
-	std::list< std::string >::iterator		iter_recent = m_list_recent.begin();
-	int i;
-	for( i = 0; i < m_recentcount; i++ )
+	std::list< std::string >::iterator		iter_recent = m_list_recent->begin();
+	bool flag_update = false;
+	for( ; iter_recent != m_list_recent->end(); ++iter_recent )
 	{
 		if( ( *iter_recent ) == path )
 		{
+			if( iter_recent == m_list_recent->begin() )
+				flag_update = true;
 			m_menu->items().erase( iter_menu );
-			m_list_recent.erase( iter_recent );
-			m_recentcount--;
+			m_list_recent->erase( iter_recent );
 			break;
 		}
 		else
 		{
-			iter_recent++;
-			iter_menu++;
+			++iter_menu;
 		}
 	}
 	// update current path if first item is deleted
-	if( i == 0 && m_recentcount > 0 )
+	if( flag_update )
 	{
-		m_label->set_label( Glib::filename_display_basename( *m_list_recent.begin() ) );
-		set_tooltip_text( *m_list_recent.begin() );
+		m_label->set_label( Glib::filename_display_basename( m_list_recent->front() ) );
+		set_tooltip_text( m_list_recent->front() );
 		m_signal_selectionchanged.emit();
 	}
 }
@@ -756,8 +998,8 @@ FilebuttonRecent::show_filechooser( void )
 	// TRANSLATORS: this is the title of filechooser dialog for selecting a new diary
 			_( "Select a Diary" ), Gtk::FILE_CHOOSER_ACTION_OPEN );
 
-	if( m_list_recent.size() > 0 )
-		m_filechooserdialog->set_filename( * m_list_recent.begin() );
+	if( m_list_recent->size() > 0 )
+		m_filechooserdialog->set_filename( * m_list_recent->begin() );
 	else
 		m_filechooserdialog->set_current_folder( Glib::get_home_dir() );
 
@@ -801,7 +1043,7 @@ FilebuttonRecent::on_drag_data_received(	const Glib::RefPtr<Gdk::DragContext>& c
 		return;
 
 	set_filename( filename );
-	std::cout << "dropped: " << filename << std::endl;
+	PRINT_DEBUG( Glib::ustring::compose( "dropped: %1", filename ) );
 
 	context->drag_finish( true, false, time );
 }
@@ -824,3 +1066,4 @@ FilebuttonRecent::signal_create_file( void )
 {
 	return m_signal_createfile;
 }
+
