@@ -20,6 +20,10 @@
 ***********************************************************************************/
 
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <gtkmm/main.h>
 
 #include "lifeograph.hpp"
@@ -33,7 +37,6 @@ main( int argc, char *argv[] )
 	bindtextdomain( PACKAGE, LOCALEDIR );
 	textdomain( PACKAGE );
 
-	Gnome::Conf::init();
 	Gtk::Main kit( argc, argv );
 
 	Glib::set_application_name( LIFEO::PROGRAM_NAME );
